'use strict';

(function (Drupal) {
  // @TODO: Capitalize on the drupal/once library instead.
  let message_initialized = false;

  Drupal.behaviors.oembedLazyloadMessageListener = {
    attach: function() {

      if (message_initialized) {
        return;
      }

      message_initialized = true;

      // Add a handler for messages emitted from the oembed security iframe.
      window.addEventListener('message', function(event) {

        // Only react to oembed-lazyload-iframe-loaded message types.
        if (event.data.hasOwnProperty('type') && event.data.type === 'oembed-lazyload-iframe-loaded') {

          // When the oembed iframe is loaded, then swap out the placeholder.
          const iframe = document.querySelector('#' + event.data.id);

          // This is a cheap hack to ensure the user doesn't see a flash as the
          // iframe and placeholder are swapped out.  Setting the opacity to a
          // value < 1 will cause the iframe to have its own stacking context.
          // This effectively means that the iframe will stack on top of the
          // button element.
          //
          // @see https://www.w3.org/TR/css-color-3/#transparency
          //
          // Important: I am assuming there's an 8 bit alpha channel!  I cannot
          // find any white-paper specification that enforces this, but it does
          // seem like a logical conclusion, as the RGB channels are 8 bit.
          iframe.style.opacity = (1 - (1 / 256));
          iframe.classList.remove('oembed-lazyload__iframe--hidden');

          // There's a 200ms transition on opacity.  At the end, we can be sure
          // that the iframe has rendered its first frame with its new stacking
          // order.  We can now detach the button safely without a 'flash'.
          setTimeout(function() {
            const button = iframe.parentNode.querySelector('.oembed-lazyload__button');
            button.classList.add('oembed-lazyload__button--hidden');

            // Restore the original full opacity as well.
            iframe.style.opacity = 1;
          }, 200);
        }
      });
    }
  };
})(Drupal);
