<?php

namespace Drupal\oembed_lazyload_vimeo\Plugin\oembed_lazyload\ProviderEnhancer;

use Drupal\Core\Render\Markup;
use Drupal\media\OEmbed\Resource;
use Drupal\oembed_lazyload\ProviderEnhancerBase;

/**
 * An implementation that enhances Vimeo content.
 *
 * @ProviderEnhancer(
 *   id = "vimeo",
 *   label = "Vimeo",
 *   providers = {
 *     "Vimeo"
 *   }
 * )
 */
class VimeoEnhancer extends ProviderEnhancerBase {

  /**
   * {@inheritdoc}
   */
  public function getPlayButton(Resource $resource) {
    $button = parent::getPlayButton($resource);

    // Replace the value with an aria-label since sighted users will see a
    // well known branding mark as the play button icon.
    $button['#attributes']['class'][] = 'oembed-lazyload__button--vimeo';
    $button['#attributes']['data-bg'] = $resource->getThumbnailUrl()->toString();
    unset($button['#value'], $button['hidden_title']);

    $title = $resource->getTitle();
    if ($title) {
      $button['#attributes']['aria-label'] = $this->t('Play video: @title', [
        '@title' => $title,
      ]);

    }
    else {
      $button['#attributes']['aria-label'] = $this->t('Play video');
    }
    $button['icon'] = [
      '#type' => 'markup',
      '#markup' => Markup::create(<<<SVG
<svg viewBox="0 0 20 20" preserveAspectRatio="xMidYMid" focusable="false" role="img">
  <polygon class="fill" points="1,0 20,10 1,20"></polygon>
</svg>
SVG
      ),
    ];
    return $button;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries() {
    $libraries = parent::getLibraries();
    $libraries[] = 'oembed_lazyload_vimeo/vimeo';
    return $libraries;
  }

  /**
   * {@inheritdoc}
   */
  public function alterOembedResponse($markup, array $options = []) {
    return str_replace('?app_id=', '?autoplay=1&app_id=', $markup);
  }

}
